# -*- coding: utf-8 -*-

"""
/***************************************************************************
 GroupStats
                                 A QGIS plugin
 Calculates stats for group of features
                              -------------------
        begin                : 2011-04-25
        copyright            : (C) 2011 by Rajmund Szostok
        email                : rayo001@interia.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""
# Import the PyQt and QGIS libraries
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from qgis.core import *
# Initialize Qt resources from file resources.py
import resources
# Import the code for the dialog
from groupstatsdialog import GroupStatsDialog

class GroupStats:

    def __init__(self, iface):
        # Save reference to the QGIS interface
        self.iface = iface
        
        #i18n
        pluginPath = QFileInfo(QgsApplication.qgisUserDbFilePath()).path()+"/python/plugins/GroupStats"
        localeName = QLocale.system().name()
        if QFileInfo(pluginPath).exists():
          self.localePath = pluginPath+"/i18n/GroupStats_" + localeName + ".qm"
        if QFileInfo(self.localePath).exists():
          self.translator = QTranslator()
          self.translator.load(self.localePath)
          if qVersion() > '4.3.3':
            QCoreApplication.installTranslator(self.translator)  

    def initGui(self):
        # Create action that will start plugin configuration
        self.action = QAction(QIcon(":/plugins/groupstats/icon.png"), \
            "Group Stats", self.iface.mainWindow())
        # connect the action to the run method
        QObject.connect(self.action, SIGNAL("triggered()"), self.run)

        # Add toolbar button and menu item
        self.iface.addToolBarIcon(self.action)
        self.iface.addPluginToMenu("&Group Stats", self.action)

    def unload(self):
        # Remove the plugin menu item and icon
        self.iface.removePluginMenu("&Group Stats",self.action)
        self.iface.removeToolBarIcon(self.action)

    # run method that performs all the real work
    def run(self):

        self.okno = GroupStatsDialog()                             # Definicja okna
        
        warstwy = QgsMapLayerRegistry.instance().mapLayers()        # Wczytanie nazw warstw z projektu
        nazwyPosortowane = warstwy.keys()
        nazwyPosortowane.sort()
        
        w_wektorowe = 0
        for id in nazwyPosortowane:
            warstwa = warstwy[id]                                    # Wczytanie nazw warstw do okna
            if warstwa.type()==0:
                self.okno.ui.warstwy.addItem(warstwa.name(), id)
                w_wektorowe = w_wektorowe +1
        
        if w_wektorowe == 0 or len(warstwy) == 0:
            QMessageBox.information(None,QCoreApplication.translate('GroupStats','Information'), \
                QCoreApplication.translate('GroupStats','Vector layers not found'))
            return
                
        QObject.connect(self.okno.ui.warstwy, SIGNAL("currentIndexChanged (int)"), self.wyborWarstwy)   # Sygnał wyboru warstwy
        QObject.connect(self.okno.ui.poleKlasyfikacji, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sygnał zmiany klasyfikatora
        QObject.connect(self.okno.ui.poleWartosci, SIGNAL("currentIndexChanged (int)"), self.zerujPostep)   # Sygnał zmiany wartosci
        QObject.connect(self.okno.ui.tylkoZaznaczone, SIGNAL("stateChanged (int)"), self.zerujPostep)   # Sygnał zmiany 'Tylko wybrane obiekty'
        QObject.connect(self.okno.ui.radioAtrybuty, SIGNAL("toggled (bool)"), self.zmianaAtrybutyGeom)   # Sygnał zmiany atrybuty/geometria
        QObject.connect(self.okno.ui.radioGeometria, SIGNAL("toggled (bool)"), self.zmianaAtrybutyGeom)   # Sygnał zmiany atrybuty/geometria
        QObject.connect(self.okno.ui.kopiuj, SIGNAL("clicked ()"), self.kopiowanie)   # Sygnał kopiowania do schowka
        QObject.connect(self.okno.ui.buttonBox, SIGNAL("accepted ()"), self.obliczenia)                 # Sygnał rozpoczęcia obliczeń
 
        self.okno.show()                                            # Pokaż okno
        self.wyborWarstwy(0)                                        # Wybierz pierwszą warstwę
        
        
    def wyborWarstwy(self, indeks, klasyfikator=0):
        
        id = self.okno.ui.warstwy.itemData(indeks)                          # Pobranie ID wybranej warstwy
        warstwa = QgsMapLayerRegistry.instance().mapLayer(id.toString())    
        provider = warstwa.dataProvider()
        atrybuty = provider.fields()                                        # Pobranie atrybutów wybranej warstwy
        
        self.okno.ui.poleKlasyfikacji.clear()                               # Wyświetlenie atrybutów wybranej warstwy
        self.okno.ui.poleWartosci.clear()
        for id, atrybut in atrybuty.iteritems():
            self.okno.ui.poleKlasyfikacji.addItem(atrybut.name(),id)
            if atrybut.typeName().toUpper() in ('REAL', 'INTEGER', 'INT4', 'INT8', 'FLOAT4') and self.okno.ui.radioAtrybuty.isChecked():
                self.okno.ui.poleWartosci.addItem(atrybut.name(),id)

        self.okno.ui.poleKlasyfikacji.setCurrentIndex(klasyfikator)

        if self.okno.ui.radioGeometria.isChecked():
            if warstwa.geometryType()==0:                               # point
                self.okno.ui.radioAtrybuty.setChecked(True)
            elif warstwa.geometryType()==1:                             # line
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Length'))
            elif warstwa.geometryType()==2:                             # polygon
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Area'))
                self.okno.ui.poleWartosci.addItem(QCoreApplication.translate('GroupStats','Perimeter'))

    def zmianaAtrybutyGeom(self, nic):
        indeks = self.okno.ui.warstwy.currentIndex()
        klasyfikator = self.okno.ui.poleKlasyfikacji.currentIndex()
        self.wyborWarstwy(indeks, klasyfikator)
    
    def obliczenia(self):
        if self.okno.ui.radioGeometria.isChecked():
            self.obliczeniaGeom()
        else:
            self.obliczeniaAtryb()
            
    def obliczeniaAtryb(self):
        indeks = self.okno.ui.warstwy.currentIndex()
        idWarstwy = self.okno.ui.warstwy.itemData(indeks)
        warstwa = QgsMapLayerRegistry.instance().mapLayer(idWarstwy.toString())
        provider = warstwa.dataProvider()
        
        if self.okno.ui.tylkoZaznaczone.isChecked():
            zaznaczoneObiektyIds=warstwa.selectedFeaturesIds()
            tylkoZaznaczone=True
        else:
            zaznaczoneObiektyIds=[]
            tylkoZaznaczone=False
        
        idKlas = self.okno.ui.poleKlasyfikacji.currentIndex()
        klasyfikator = self.okno.ui.poleKlasyfikacji.itemData(idKlas).toInt()[0]
        
        idWart = self.okno.ui.poleWartosci.currentIndex()
        wartosc = self.okno.ui.poleWartosci.itemData(idWart).toInt()[0]
        
        atrybutyWszy = provider.attributeIndexes()
        provider.select(atrybutyWszy)
        f=QgsFeature()                                          # Wyszukiwanie właściwe
        wyniki = {}
        licznik_caly = provider.featureCount()
        #QMessageBox.information(None,"Uwaga!", \
        #    `int(licznik_caly)`)
        licznik = licznik_caly

        while provider.nextFeature(f):                          # Wyszukaj obiekty zawierające numer w numerze
            if (tylkoZaznaczone==False) or (tylkoZaznaczone and (f.id() in zaznaczoneObiektyIds)):
                if wyniki.has_key(f.attributeMap()[klasyfikator].toString()):
                    wyniki[f.attributeMap()[klasyfikator].toString()].append(f.attributeMap()[wartosc].toReal()[0])
                else:
                    wyniki[f.attributeMap()[klasyfikator].toString()] = [f.attributeMap()[wartosc].toReal()[0]]
            
            licznik = licznik-1
            self.okno.ui.postep.setValue(int(100*(licznik_caly-licznik)/licznik_caly))
            
        
        
        for n in range(self.okno.ui.wyniki.rowCount(),-1,-1):      # Wyczyść tabelę wyników
            self.okno.ui.wyniki.removeRow(n)
        
        self.okno.ui.wyniki.setSortingEnabled (0)
        for grupa, wartosci in wyniki.iteritems():                                         # Wpisz wyniki do tabeli
            self.okno.ui.wyniki.insertRow(0)
            a=QTableWidgetItem(0)
            a.setText(grupa)
            self.okno.ui.wyniki.setItem(0,0,a)
                        
            b=QTableWidgetItem(0)
            b.setText(`len(wyniki[grupa])`)
            b.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,1,b)
            
            c=QTableWidgetItem(0)
            c.setText("%.2f" % sum(wyniki[grupa]))
            c.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,2,c)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % (sum(wyniki[grupa])/len(wyniki[grupa])))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,3,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % min(wyniki[grupa]))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,4,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % max(wyniki[grupa]))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,5,d)
        self.okno.ui.wyniki.setSortingEnabled (1)
        
        for i in range(6):
            self.okno.ui.wyniki.resizeColumnToContents (i)

    def obliczeniaGeom(self):
        indeks = self.okno.ui.warstwy.currentIndex()
        idWarstwy = self.okno.ui.warstwy.itemData(indeks)
        warstwa = QgsMapLayerRegistry.instance().mapLayer(idWarstwy.toString())
        provider = warstwa.dataProvider()
   
        if self.okno.ui.tylkoZaznaczone.isChecked():
            zaznaczoneObiektyIds=warstwa.selectedFeaturesIds()
            tylkoZaznaczone=True
        else:
            zaznaczoneObiektyIds=[]
            tylkoZaznaczone=False
   
        
        idKlas = self.okno.ui.poleKlasyfikacji.currentIndex()
        klasyfikator = self.okno.ui.poleKlasyfikacji.itemData(idKlas).toInt()[0]
        
        wartosc = self.okno.ui.poleWartosci.currentText()
        #wartosc = self.okno.ui.poleWartosci.itemData(idWart).toInt()[0]
        
        atrybutyWszy = provider.attributeIndexes()
        provider.select(atrybutyWszy)
        f=QgsFeature()                                          # Wyszukiwanie właściwe
        wyniki = {}
        licznik_caly = provider.featureCount()
        #QMessageBox.information(None,"Uwaga!", \
        #    `int(licznik_caly)`)
        licznik = licznik_caly
        
        

        if wartosc == QCoreApplication.translate('GroupStats','Length') or wartosc == QCoreApplication.translate('GroupStats','Perimeter'):
            while provider.nextFeature(f):                          # Wyszukaj obiekty zawierające numer w numerze
                if (tylkoZaznaczone==False) or (tylkoZaznaczone and (f.id() in zaznaczoneObiektyIds)):
                    if wyniki.has_key(f.attributeMap()[klasyfikator].toString()):
                        wyniki[f.attributeMap()[klasyfikator].toString()].append(f.geometry().length())
                    else:
                        wyniki[f.attributeMap()[klasyfikator].toString()] = [f.geometry().length()]
                licznik = licznik-1
                self.okno.ui.postep.setValue(int(100*(licznik_caly-licznik)/licznik_caly))
                
        elif wartosc == QCoreApplication.translate('GroupStats','Area'):
            while provider.nextFeature(f):                          # Wyszukaj obiekty zawierające numer w numerze
                if (tylkoZaznaczone==False) or (tylkoZaznaczone and (f.id() in zaznaczoneObiektyIds)):
                    if wyniki.has_key(f.attributeMap()[klasyfikator].toString()):
                        wyniki[f.attributeMap()[klasyfikator].toString()].append(f.geometry().area())
                    else:
                        wyniki[f.attributeMap()[klasyfikator].toString()] = [f.geometry().area()]
                licznik = licznik-1
                self.okno.ui.postep.setValue(int(100*(licznik_caly-licznik)/licznik_caly))
        #elif wartosc == u'Obwód':
        #    while provider.nextFeature(f):                          # Wyszukaj obiekty zawierające numer w numerze     liczyc obwod!!!!!!!!
        #        if wyniki.has_key(f.attributeMap()[klasyfikator].toString()):
        #            wyniki[f.attributeMap()[klasyfikator].toString()].append(f.geometry().length())
        #        else:
        #            wyniki[f.attributeMap()[klasyfikator].toString()] = [f.geometry().length()]
        #        licznik = licznik-1
        #        self.okno.ui.postep.setValue(int(100*(licznik_caly-licznik)/licznik_caly))
            
        
        
        for n in range(self.okno.ui.wyniki.rowCount(),-1,-1):      # Wyczyść tabelę wyników
            self.okno.ui.wyniki.removeRow(n)
        
        self.okno.ui.wyniki.setSortingEnabled (0)
        for grupa, wartosci in wyniki.iteritems():                                         # Wpisz wyniki do tabeli
            self.okno.ui.wyniki.insertRow(0)
            a=QTableWidgetItem(0)
            a.setText(grupa)
            self.okno.ui.wyniki.setItem(0,0,a)
                        
            b=QTableWidgetItem(0)
            b.setText(`len(wyniki[grupa])`)
            b.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,1,b)
            
            c=QTableWidgetItem(0)
            c.setText("%.2f" % sum(wyniki[grupa]))
            c.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,2,c)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % (sum(wyniki[grupa])/len(wyniki[grupa])))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,3,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % min(wyniki[grupa]))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,4,d)
            
            d=QTableWidgetItem(0)
            d.setText("%.2f" % max(wyniki[grupa]))
            d.setTextAlignment(0x0082)
            self.okno.ui.wyniki.setItem(0,5,d)
        self.okno.ui.wyniki.setSortingEnabled (1)
        
        for i in range(6):
            self.okno.ui.wyniki.resizeColumnToContents (i)
    
    def zerujPostep(self, nic):
        self.okno.ui.postep.setValue(0)
        self.okno.ui.wyniki.clearContents()
        
    def kopiowanie (self):
        if len(self.okno.ui.wyniki.selectedItems()) == 0:
            return
        
        schowek = QApplication.clipboard()
        tekst = ''
        liczbaKolumn = self.okno.ui.wyniki.columnCount()
        wierszPocz = self.okno.ui.wyniki.selectedRanges()[0].topRow()
        wierszKonc = self.okno.ui.wyniki.selectedRanges()[0].bottomRow()
        
        for i in range(liczbaKolumn):
            tekst = tekst + self.okno.ui.wyniki.horizontalHeaderItem(i).text()
            if i != (liczbaKolumn-1):
                    tekst = tekst + chr(9)
        tekst = tekst + chr(13)
        
        for i in range(wierszPocz, wierszKonc+1):
            for j in range(liczbaKolumn):
                tekst = tekst + self.okno.ui.wyniki.item(i,j).text()
                if j != (liczbaKolumn-1):
                    tekst = tekst + chr(9)
            if i != wierszKonc:
                tekst = tekst + chr(13)
                
        schowek.setText(tekst)